/****************************************************************************
 *                                                                          *
 *                      Velena Source Code V1.0                             *
 *                   Written by Giuliano Bertoletti                         *
 *       Based on the knowledged approach of Louis Victor Allis             *
 *   Copyright (C) 1996-97 by Giuliano Bertoletti & GBE 32241 Software PR   *
 *                                                                          *
 ****************************************************************************

 Portable engine version.
 read the README file for further informations.

 ============================================================================

 Changes have been made to this code for inclusion with Gnect. It is
 released under the GNU General Public License with Giuliano's approval.
 The original and complete Velena Engine source code can be found at:

 http://www.ce.unipr.it/~gbe/velena.html

*/


#define DEPTH 4

#define IMPOSSIBLE -30000

#define POSITIVE   1

#define BADMOVE   -16384
#define NOCOMMENT      0
#define GOODMOVE   16384

#define BRIGHTMOVE (GOODMOVE/2)
#define SMARTMOVE  (BADMOVE/2)

#define RANDVARIANCE 35
