/* lightsoff.c generated by valac 0.16.0, the Vala compiler
 * generated from lightsoff.vala, do not modify */


#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gio/gio.h>
#include <clutter/clutter.h>
#include <glib/gi18n-lib.h>
#include <clutter-gtk/clutter-gtk.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include <games-stock.h>
#include <locale.h>


#define TYPE_LIGHTS_OFF (lights_off_get_type ())
#define LIGHTS_OFF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LIGHTS_OFF, LightsOff))
#define LIGHTS_OFF_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LIGHTS_OFF, LightsOffClass))
#define IS_LIGHTS_OFF(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LIGHTS_OFF))
#define IS_LIGHTS_OFF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LIGHTS_OFF))
#define LIGHTS_OFF_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LIGHTS_OFF, LightsOffClass))

typedef struct _LightsOff LightsOff;
typedef struct _LightsOffClass LightsOffClass;
typedef struct _LightsOffPrivate LightsOffPrivate;

#define TYPE_GAME_VIEW (game_view_get_type ())
#define GAME_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GAME_VIEW, GameView))
#define GAME_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GAME_VIEW, GameViewClass))
#define IS_GAME_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GAME_VIEW))
#define IS_GAME_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GAME_VIEW))
#define GAME_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GAME_VIEW, GameViewClass))

typedef struct _GameView GameView;
typedef struct _GameViewClass GameViewClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _LightsOff {
	GtkApplication parent_instance;
	LightsOffPrivate * priv;
};

struct _LightsOffClass {
	GtkApplicationClass parent_class;
};

struct _LightsOffPrivate {
	GSettings* settings;
	GtkWindow* window;
	GameView* game_view;
};


static gpointer lights_off_parent_class = NULL;

GType lights_off_get_type (void) G_GNUC_CONST;
GType game_view_get_type (void) G_GNUC_CONST;
#define LIGHTS_OFF_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_LIGHTS_OFF, LightsOffPrivate))
enum  {
	LIGHTS_OFF_DUMMY_PROPERTY
};
static void lights_off_new_game_cb (LightsOff* self);
static void _lights_off_new_game_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self);
static void lights_off_quit_cb (LightsOff* self);
static void _lights_off_quit_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self);
static void lights_off_help_cb (LightsOff* self);
static void _lights_off_help_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self);
static void lights_off_about_cb (LightsOff* self);
static void _lights_off_about_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self);
static LightsOff* lights_off_new (void);
static LightsOff* lights_off_construct (GType object_type);
static void lights_off_real_startup (GApplication* base);
static gboolean lights_off_key_release_event_cb (LightsOff* self, ClutterActor* actor, ClutterKeyEvent* event);
static gboolean _lights_off_key_release_event_cb_clutter_actor_key_release_event (ClutterActor* _sender, ClutterKeyEvent* event, gpointer self);
GameView* game_view_new (gint level);
GameView* game_view_construct (GType object_type, gint level);
static void lights_off_level_changed_cb (LightsOff* self, gint level);
static void _lights_off_level_changed_cb_game_view_level_changed (GameView* _sender, gint level, gpointer self);
void game_view_hide_cursor (GameView* self);
void game_view_move_cursor (GameView* self, gint x_step, gint y_step);
void game_view_activate_cursor (GameView* self);
static void lights_off_real_activate (GApplication* base);
void game_view_reset_game (GameView* self);
gint lights_off_main (gchar** args, int args_length1);
static void lights_off_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GActionEntry LIGHTS_OFF_action_entries[4] = {{"new-game", _lights_off_new_game_cb_gsimple_action_activate_callback}, {"quit", _lights_off_quit_cb_gsimple_action_activate_callback}, {"help", _lights_off_help_cb_gsimple_action_activate_callback}, {"about", _lights_off_about_cb_gsimple_action_activate_callback}};

static void _lights_off_new_game_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self) {
	lights_off_new_game_cb (self);
}


static void _lights_off_quit_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self) {
	lights_off_quit_cb (self);
}


static void _lights_off_help_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self) {
	lights_off_help_cb (self);
}


static void _lights_off_about_cb_gsimple_action_activate_callback (GSimpleAction* action, GVariant* parameter, gpointer self) {
	lights_off_about_cb (self);
}


static LightsOff* lights_off_construct (GType object_type) {
	LightsOff * self = NULL;
	self = (LightsOff*) g_object_new (object_type, "application-id", "org.gnome.lightsoff", "flags", G_APPLICATION_FLAGS_NONE, NULL);
	return self;
}


static LightsOff* lights_off_new (void) {
	return lights_off_construct (TYPE_LIGHTS_OFF);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static gboolean _lights_off_key_release_event_cb_clutter_actor_key_release_event (ClutterActor* _sender, ClutterKeyEvent* event, gpointer self) {
	gboolean result;
	result = lights_off_key_release_event_cb (self, _sender, event);
	return result;
}


static void _lights_off_level_changed_cb_game_view_level_changed (GameView* _sender, gint level, gpointer self) {
	lights_off_level_changed_cb (self, level);
}


static void lights_off_real_startup (GApplication* base) {
	LightsOff * self;
	GMenu* _tmp0_;
	GMenu* menu;
	GMenu* _tmp1_;
	GMenu* section;
	GMenu* _tmp2_;
	GMenu* _tmp3_;
	const gchar* _tmp4_ = NULL;
	GMenu* _tmp5_;
	GMenu* _tmp6_;
	GMenu* _tmp7_;
	const gchar* _tmp8_ = NULL;
	GMenu* _tmp9_;
	const gchar* _tmp10_ = NULL;
	GMenu* _tmp11_;
	GMenu* _tmp12_;
	GMenu* _tmp13_;
	const gchar* _tmp14_ = NULL;
	GSettings* _tmp15_;
	GtkApplicationWindow* _tmp16_;
	GtkWindow* _tmp17_;
	GtkClutterEmbed* _tmp18_;
	GtkClutterEmbed* _tmp19_;
	GtkClutterEmbed* clutter_embed;
	GtkWindow* _tmp20_;
	ClutterActor* _tmp21_ = NULL;
	ClutterStage* _tmp22_;
	ClutterStage* stage;
	ClutterColor _tmp23_ = {0};
	GSettings* _tmp24_;
	gint _tmp25_ = 0;
	GameView* _tmp26_;
	GameView* _tmp27_;
	GameView* _tmp28_;
	GameView* _tmp29_;
	GameView* _tmp30_;
	GameView* _tmp31_;
	gfloat _tmp32_;
	gfloat _tmp33_;
	GameView* _tmp34_;
	gfloat _tmp35_;
	gfloat _tmp36_;
	gfloat _tmp37_;
	gfloat _tmp38_;
	gfloat _tmp39_;
	gfloat _tmp40_;
	self = (LightsOff*) base;
	G_APPLICATION_CLASS (lights_off_parent_class)->startup ((GApplication*) GTK_APPLICATION (self));
	g_action_map_add_action_entries ((GActionMap*) self, LIGHTS_OFF_action_entries, G_N_ELEMENTS (LIGHTS_OFF_action_entries), self);
	_tmp0_ = g_menu_new ();
	menu = _tmp0_;
	_tmp1_ = g_menu_new ();
	section = _tmp1_;
	_tmp2_ = section;
	g_menu_append_section (menu, NULL, (GMenuModel*) _tmp2_);
	_tmp3_ = section;
	_tmp4_ = _ ("_New Game");
	g_menu_append (_tmp3_, _tmp4_, "app.new-game");
	_tmp5_ = g_menu_new ();
	_g_object_unref0 (section);
	section = _tmp5_;
	_tmp6_ = section;
	g_menu_append_section (menu, NULL, (GMenuModel*) _tmp6_);
	_tmp7_ = section;
	_tmp8_ = _ ("_Help");
	g_menu_append (_tmp7_, _tmp8_, "app.help");
	_tmp9_ = section;
	_tmp10_ = _ ("_About");
	g_menu_append (_tmp9_, _tmp10_, "app.about");
	_tmp11_ = g_menu_new ();
	_g_object_unref0 (section);
	section = _tmp11_;
	_tmp12_ = section;
	g_menu_append_section (menu, NULL, (GMenuModel*) _tmp12_);
	_tmp13_ = section;
	_tmp14_ = _ ("_Quit");
	g_menu_append (_tmp13_, _tmp14_, "app.quit");
	gtk_application_set_app_menu ((GtkApplication*) self, (GMenuModel*) menu);
	_tmp15_ = g_settings_new ("org.gnome.lightsoff");
	_g_object_unref0 (self->priv->settings);
	self->priv->settings = _tmp15_;
	_tmp16_ = (GtkApplicationWindow*) gtk_application_window_new ((GtkApplication*) self);
	_tmp17_ = (GtkWindow*) g_object_ref_sink (_tmp16_);
	_g_object_unref0 (self->priv->window);
	self->priv->window = _tmp17_;
	_tmp18_ = (GtkClutterEmbed*) gtk_clutter_embed_new ();
	_tmp19_ = g_object_ref_sink (_tmp18_);
	clutter_embed = _tmp19_;
	gtk_widget_show ((GtkWidget*) clutter_embed);
	_tmp20_ = self->priv->window;
	gtk_container_add ((GtkContainer*) _tmp20_, (GtkWidget*) clutter_embed);
	_tmp21_ = gtk_clutter_embed_get_stage (clutter_embed);
	_tmp22_ = _g_object_ref0 (CLUTTER_STAGE (_tmp21_));
	stage = _tmp22_;
	g_signal_connect_object ((ClutterActor*) stage, "key-release-event", (GCallback) _lights_off_key_release_event_cb_clutter_actor_key_release_event, self, 0);
	clutter_color_from_string (&_tmp23_, "#000000");
	clutter_stage_set_color (stage, &_tmp23_);
	_tmp24_ = self->priv->settings;
	_tmp25_ = g_settings_get_int (_tmp24_, "level");
	_tmp26_ = game_view_new (_tmp25_);
	_tmp27_ = g_object_ref_sink (_tmp26_);
	_g_object_unref0 (self->priv->game_view);
	self->priv->game_view = _tmp27_;
	_tmp28_ = self->priv->game_view;
	g_signal_connect_object (_tmp28_, "level-changed", (GCallback) _lights_off_level_changed_cb_game_view_level_changed, self, 0);
	_tmp29_ = self->priv->game_view;
	clutter_actor_show ((ClutterActor*) _tmp29_);
	_tmp30_ = self->priv->game_view;
	clutter_container_add_actor ((ClutterContainer*) stage, (ClutterActor*) _tmp30_);
	_tmp31_ = self->priv->game_view;
	_tmp32_ = clutter_actor_get_width ((ClutterActor*) _tmp31_);
	_tmp33_ = _tmp32_;
	_tmp34_ = self->priv->game_view;
	_tmp35_ = clutter_actor_get_height ((ClutterActor*) _tmp34_);
	_tmp36_ = _tmp35_;
	clutter_actor_set_size ((ClutterActor*) stage, _tmp33_, _tmp36_);
	_tmp37_ = clutter_actor_get_width ((ClutterActor*) stage);
	_tmp38_ = _tmp37_;
	_tmp39_ = clutter_actor_get_height ((ClutterActor*) stage);
	_tmp40_ = _tmp39_;
	gtk_widget_set_size_request ((GtkWidget*) clutter_embed, (gint) _tmp38_, (gint) _tmp40_);
	_g_object_unref0 (stage);
	_g_object_unref0 (clutter_embed);
	_g_object_unref0 (section);
	_g_object_unref0 (menu);
}


static void lights_off_level_changed_cb (LightsOff* self, gint level) {
	GSettings* _tmp0_;
	gint _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->settings;
	_tmp1_ = level;
	g_settings_set_int (_tmp0_, "level", _tmp1_);
}


static gboolean lights_off_key_release_event_cb (LightsOff* self, ClutterActor* actor, ClutterKeyEvent* event) {
	gboolean result = FALSE;
	ClutterKeyEvent _tmp0_;
	guint _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (actor != NULL, FALSE);
	g_return_val_if_fail (event != NULL, FALSE);
	_tmp0_ = *event;
	_tmp1_ = _tmp0_.keyval;
	switch (_tmp1_) {
		case CLUTTER_KEY_Escape:
		{
			GameView* _tmp2_;
			_tmp2_ = self->priv->game_view;
			game_view_hide_cursor (_tmp2_);
			result = TRUE;
			return result;
		}
		case CLUTTER_KEY_Down:
		{
			GameView* _tmp3_;
			_tmp3_ = self->priv->game_view;
			game_view_move_cursor (_tmp3_, 0, 1);
			result = TRUE;
			return result;
		}
		case CLUTTER_KEY_Up:
		{
			GameView* _tmp4_;
			_tmp4_ = self->priv->game_view;
			game_view_move_cursor (_tmp4_, 0, -1);
			result = TRUE;
			return result;
		}
		case CLUTTER_KEY_Left:
		{
			GameView* _tmp5_;
			_tmp5_ = self->priv->game_view;
			game_view_move_cursor (_tmp5_, -1, 0);
			result = TRUE;
			return result;
		}
		case CLUTTER_KEY_Right:
		{
			GameView* _tmp6_;
			_tmp6_ = self->priv->game_view;
			game_view_move_cursor (_tmp6_, 1, 0);
			result = TRUE;
			return result;
		}
		case CLUTTER_KEY_Return:
		{
			GameView* _tmp7_;
			_tmp7_ = self->priv->game_view;
			game_view_activate_cursor (_tmp7_);
			result = TRUE;
			return result;
		}
		default:
		{
			result = FALSE;
			return result;
		}
	}
}


static void lights_off_real_activate (GApplication* base) {
	LightsOff * self;
	GtkWindow* _tmp0_;
	self = (LightsOff*) base;
	_tmp0_ = self->priv->window;
	gtk_window_present (_tmp0_);
}


static void lights_off_new_game_cb (LightsOff* self) {
	GameView* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->game_view;
	game_view_reset_game (_tmp0_);
}


static void lights_off_quit_cb (LightsOff* self) {
	GtkWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->window;
	gtk_widget_destroy ((GtkWidget*) _tmp0_);
}


static void lights_off_help_cb (LightsOff* self) {
	GError * _inner_error_ = NULL;
	g_return_if_fail (self != NULL);
	{
		GtkWindow* _tmp0_;
		GdkScreen* _tmp1_ = NULL;
		guint32 _tmp2_ = 0U;
		_tmp0_ = self->priv->window;
		_tmp1_ = gtk_window_get_screen (_tmp0_);
		_tmp2_ = gtk_get_current_event_time ();
		gtk_show_uri (_tmp1_, "help:lightsoff", _tmp2_, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch0_g_error;
		}
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* e = NULL;
		GError* _tmp3_;
		const gchar* _tmp4_;
		e = _inner_error_;
		_inner_error_ = NULL;
		_tmp3_ = e;
		_tmp4_ = _tmp3_->message;
		g_warning ("lightsoff.vala:115: Failed to show help: %s", _tmp4_);
		_g_error_free0 (e);
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
		g_clear_error (&_inner_error_);
		return;
	}
}


static void lights_off_about_cb (LightsOff* self) {
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar** _tmp2_ = NULL;
	gchar** authors;
	gint authors_length1;
	gint _authors_size_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar** _tmp5_ = NULL;
	gchar** artists;
	gint artists_length1;
	gint _artists_size_;
	gchar* _tmp6_;
	gchar** _tmp7_ = NULL;
	gchar** documenters;
	gint documenters_length1;
	gint _documenters_size_;
	GtkWindow* _tmp8_;
	const gchar* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
	const gchar* _tmp11_ = NULL;
	gchar* _tmp12_ = NULL;
	gchar* _tmp13_;
	const gchar* _tmp14_ = NULL;
	const gchar* _tmp15_ = NULL;
	g_return_if_fail (self != NULL);
	_tmp0_ = g_strdup ("Tim Horton");
	_tmp1_ = g_strdup ("Robert Ancell");
	_tmp2_ = g_new0 (gchar*, 3 + 1);
	_tmp2_[0] = _tmp0_;
	_tmp2_[1] = _tmp1_;
	_tmp2_[2] = NULL;
	authors = _tmp2_;
	authors_length1 = 3;
	_authors_size_ = authors_length1;
	_tmp3_ = g_strdup ("Tim Horton");
	_tmp4_ = g_strdup ("Ulisse Perusin");
	_tmp5_ = g_new0 (gchar*, 3 + 1);
	_tmp5_[0] = _tmp3_;
	_tmp5_[1] = _tmp4_;
	_tmp5_[2] = NULL;
	artists = _tmp5_;
	artists_length1 = 3;
	_artists_size_ = artists_length1;
	_tmp6_ = g_strdup ("Eric Baudais");
	_tmp7_ = g_new0 (gchar*, 2 + 1);
	_tmp7_[0] = _tmp6_;
	_tmp7_[1] = NULL;
	documenters = _tmp7_;
	documenters_length1 = 2;
	_documenters_size_ = documenters_length1;
	_tmp8_ = self->priv->window;
	_tmp9_ = _ ("Lights Off");
	_tmp10_ = _ ("Turn off all the lights\n\nLights Off is a part of GNOME Games.");
	_tmp11_ = _ ("Lights Off");
	_tmp12_ = games_get_license (_tmp11_);
	_tmp13_ = _tmp12_;
	_tmp14_ = _ ("translator-credits");
	_tmp15_ = _ ("GNOME Games web site");
	gtk_show_about_dialog (_tmp8_, "program-name", _tmp9_, "version", VERSION, "comments", _tmp10_, "copyright", "Copyright © 2009 Tim Horton", "license", _tmp13_, "wrap-license", TRUE, "authors", authors, "artists", artists, "documenters", documenters, "translator-credits", _tmp14_, "logo-icon-name", "lightsoff", "website", "http://www.gnome.org/projects/gnome-games", "website-label", _tmp15_, NULL, NULL);
	_g_free0 (_tmp13_);
	documenters = (_vala_array_free (documenters, documenters_length1, (GDestroyNotify) g_free), NULL);
	artists = (_vala_array_free (artists, artists_length1, (GDestroyNotify) g_free), NULL);
	authors = (_vala_array_free (authors, authors_length1, (GDestroyNotify) g_free), NULL);
}


gint lights_off_main (gchar** args, int args_length1) {
	gint result = 0;
	const gchar* _tmp0_ = NULL;
	ClutterInitError _tmp1_ = 0;
	LightsOff* _tmp2_;
	LightsOff* app;
	LightsOff* _tmp3_;
	gint _tmp4_ = 0;
	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
	_tmp0_ = _ ("Lights Off");
	g_set_application_name (_tmp0_);
	_tmp1_ = gtk_clutter_init (&args_length1, &args);
	if (_tmp1_ != CLUTTER_INIT_SUCCESS) {
		g_warning ("lightsoff.vala:170: Failed to initialise Clutter");
		result = EXIT_FAILURE;
		return result;
	}
	games_stock_init ();
	_tmp2_ = lights_off_new ();
	app = _tmp2_;
	_tmp3_ = app;
	_tmp4_ = g_application_run ((GApplication*) _tmp3_, 0, NULL);
	result = _tmp4_;
	_g_object_unref0 (app);
	return result;
}


int main (int argc, char ** argv) {
	g_type_init ();
	return lights_off_main (argv, argc);
}


static void lights_off_class_init (LightsOffClass * klass) {
	lights_off_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (LightsOffPrivate));
	G_APPLICATION_CLASS (klass)->startup = lights_off_real_startup;
	G_APPLICATION_CLASS (klass)->activate = lights_off_real_activate;
	G_OBJECT_CLASS (klass)->finalize = lights_off_finalize;
}


static void lights_off_instance_init (LightsOff * self) {
	self->priv = LIGHTS_OFF_GET_PRIVATE (self);
}


static void lights_off_finalize (GObject* obj) {
	LightsOff * self;
	self = LIGHTS_OFF (obj);
	_g_object_unref0 (self->priv->settings);
	_g_object_unref0 (self->priv->window);
	_g_object_unref0 (self->priv->game_view);
	G_OBJECT_CLASS (lights_off_parent_class)->finalize (obj);
}


GType lights_off_get_type (void) {
	static volatile gsize lights_off_type_id__volatile = 0;
	if (g_once_init_enter (&lights_off_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (LightsOffClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) lights_off_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (LightsOff), 0, (GInstanceInitFunc) lights_off_instance_init, NULL };
		GType lights_off_type_id;
		lights_off_type_id = g_type_register_static (GTK_TYPE_APPLICATION, "LightsOff", &g_define_type_info, 0);
		g_once_init_leave (&lights_off_type_id__volatile, lights_off_type_id);
	}
	return lights_off_type_id__volatile;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



