/*
 * Translatable strings file generated by get_titles.pl.
 * Add this file to your project's POTFILES.in.
 * DO NOT compile it as part of your application.
 */
NC_("mahjongg map name", "The Ziggurat")
NC_("mahjongg map name", "Four Bridges")
NC_("mahjongg map name", "Cloud")
NC_("mahjongg map name", "Tic-Tac-Toe")
NC_("mahjongg map name", "Red Dragon")
NC_("mahjongg map name", "Pyramid's Walls")
NC_("mahjongg map name", "Confounding Cross")
NC_("mahjongg map name", "Difficult")
